from __future__ import annotations
import abc
import copy
import dataclasses
from dataclasses import dataclass
from typing import Union, List, Any, Tuple

import numpy as np

import commonroad.geometry.transform
from commonroad.common.util import Interval, AngleInterval, make_valid_orientation
from commonroad.common.validity import is_real_number_vector, is_real_number, is_valid_orientation, ValidTypes
from commonroad.geometry.shape import Shape
from commonroad.visualization.draw_params import OptionalSpecificOrAllDrawParams, StateParams
from commonroad.visualization.renderer import IRenderer

FloatExactOrInterval = Union[float, Interval]
AngleExactOrInterval = Union[float, AngleInterval]
ExactOrShape = Union[np.ndarray, Shape]


@dataclass
class State(abc.ABC):
    """
    This is a class representing the Base State.

    :param time_step: Discrete time step :math:`t_i`
    """
    time_step: Union[int, Interval] = None

    def __eq__(self, other: State):
        if not isinstance(other, State):
            return False
        if set(self.attributes) != set(other.attributes):
            return False

        dec = 10
        for attr in self.attributes:
            val_self = getattr(self, attr)
            val_other = getattr(other, attr)

            if attr == 'position' and (isinstance(val_self, np.ndarray) or isinstance(val_other, np.ndarray)):
                if isinstance(val_self, np.ndarray) and isinstance(val_other, np.ndarray):
                    val_self = tuple(np.around(self.position.astype(float), dec))
                    val_other = tuple(np.around(self.position.astype(float), dec))
                else:
                    return False

            if isinstance(val_self, float):
                val_self = round(val_self, dec)
            if isinstance(val_other, float):
                val_other = round(val_other, dec)

            if val_self != val_other:
                return False

        return True

    def __hash__(self):
        values = list()

        dec = 10
        for attr in self.attributes:
            val = getattr(self, attr)

            if attr == 'position' and isinstance(self.position, np.ndarray):
                val = tuple(np.around(self.position.astype(float), dec))

            if isinstance(val, float):
                val = round(val, dec)
            values.append(val)
        return hash(tuple(values))

    def __array__(self: Union[TraceState]) -> np.ndarray:
        """
        Converts the State into a 1D-numpy array by iterating over all fields of the dataclass. The order of the fields
        as defined in the dataclass is preserved.
        Note: Time step is not included, as the numpy array only contains the state vector.
        """
        values = list()
        for field in dataclasses.fields(self):
            if field.name == "time_step":
                # time step not included in state array
                continue
            elif field.name == "position":
                values.append(getattr(self, field.name)[0])  # x-position
                values.append(getattr(self, field.name)[1])  # y-position
            else:
                values.append(getattr(self, field.name))
        return np.array(values)

    @property
    def attributes(self) -> List[str]:
        """
        Returns all attributes used in state space.

        Attributes
        """
        fields = self.__dict__

        return [field_name for field_name in fields]

    @property
    def used_attributes(self) -> List[str]:
        """
        Returns all initialized attributed in state space.

        Initialized attributes
        """
        used_fields = list()
        for field_name in self.attributes:
            if getattr(self, field_name) is not None:
                used_fields.append(field_name)

        return used_fields

    @property
    def is_uncertain_position(self) -> bool:
        """
        Checks whether the position is uncertain.

        Uncertain or not
        """
        if hasattr(self, "position"):
            return isinstance(getattr(self, "position"), Shape)
        return False

    @property
    def is_uncertain_orientation(self):
        """
        Checks whether the orientation is uncertain.

        Uncertain or not
        """
        if hasattr(self, "orientation"):
            return isinstance(getattr(self, "orientation"), AngleInterval)
        return False

    def has_value(self, attr: str) -> bool:
        """
        Checks whether an attribute is given and is initialized with a value.

        :param attr: Name of attribute
        """
        return hasattr(self, attr) and getattr(self, attr) is not None

    def translate_rotate(self, translation: np.ndarray, angle: float) -> TraceState:
        """
        First translates the state and then rotates the state around the origin.

        :param translation: Translation vector [x_off, y_off] in x- and y-direction
        :param angle: Rotation angle in radian (counter-clockwise)
        :return: Transformed state
        """
        assert is_real_number_vector(translation, 2), ('<State/translate_rotate>: argument translation is not '
                                                       'a vector of real numbers of length 2.')
        assert is_real_number(angle), ('<State/translate_rotate>: argument angle must be a scalar. '
                                       'angle = %s' % angle)
        assert is_valid_orientation(angle), ('<State/translate_rotate>: argument angle must be within the '
                                             'interval [-2pi,2pi]. angle = %s.' % angle)

        transformed_state = copy.copy(self)
        if hasattr(self, "position") and getattr(self, "position") is not None:
            if isinstance(self.position, ValidTypes.ARRAY):
                transformed_state.position = commonroad.geometry.transform.translate_rotate(np.array([self.position]),
                                                                                            translation, angle)[0]
            elif isinstance(self.position, Shape):
                transformed_state.position = self.position.translate_rotate(translation, angle)
            else:
                raise TypeError('<State/translate_rotate> Expected instance of %s or %s. Got %s instead.'
                                % (ValidTypes.ARRAY, Shape, self.position.__class__))

        if hasattr(self, "orientation") and getattr(self, "orientation") is not None:
            if isinstance(self.orientation, ValidTypes.NUMBERS):
                transformed_state.orientation = make_valid_orientation(self.orientation + angle)
            elif isinstance(self.orientation, AngleInterval):
                transformed_state.orientation = transformed_state.orientation + angle
            else:
                raise TypeError('<State/translate_rotate> Expected instance of %s or %s. Got %s instead.'
                                % (ValidTypes.NUMBERS, AngleInterval, self.orientation.__class__))

        return transformed_state

    def convert_state_to_state(self, state: TraceState) -> TraceState:
        """
        Converts state to state from different state types.

        :param state: State for converting
        """
        from_fields = dataclasses.fields(type(self))
        to_fields = dataclasses.fields(type(state))
        for from_field in from_fields:
            for to_field in to_fields:
                if from_field.name == to_field.name:
                    setattr(state, to_field.name, getattr(self, from_field.name))

        return state

    def fill_with_defaults(self):
        """
        Fills all state fields with default values.

        """
        for field in self.attributes:
            if getattr(self, field) is not None:
                continue

            if field == 'position':
                setattr(self, field, np.array([0., 0.]))
            else:
                setattr(self, field, 0.)

    def draw(self, renderer: IRenderer, draw_params: OptionalSpecificOrAllDrawParams[StateParams] = None):
        """
        Draws state.

        :param renderer: Renderer
        :param draw_params: Params
        """
        renderer.draw_state(self, draw_params)


@dataclass(eq=False)
class InitialState(State):
    """
    This is a class representing the Initial State.

    :param position: Position :math:`s_x`- and :math:`s_y` in a global coordinate system
    :param orientation: Yaw angle :math:`\\Psi`
    :param velocity: Velocity :math:`v_x` in longitudinal direction
    :param acceleration: Acceleration :math:`a_x`
    :param yaw_rate: Yaw rate :math:`\\dot{\\Psi}`
    :param slip_angle: Slip angle :math:`\\beta`
    """
    position: ExactOrShape = None
    orientation: AngleExactOrInterval = None
    velocity: FloatExactOrInterval = None
    acceleration: FloatExactOrInterval = None
    yaw_rate: FloatExactOrInterval = None
    slip_angle: FloatExactOrInterval = None


@dataclass(eq=False)
class PMState(State):
    """
    This is a class representing Point Mass State (PM State).

    :param position: Position :math:`s_x`- and :math:`s_y` in a global coordinate system
    :param velocity: Velocity :math:`v_x` in longitudinal direction
    :param velocity_y: Velocity :math:`v_x` in lateral direction
    """
    position: ExactOrShape = None
    velocity: FloatExactOrInterval = None
    velocity_y: FloatExactOrInterval = None


@dataclass(eq=False)
class KSState(State):
    """
    This is a class representing Kinematic Single Track State (KS State).

    :param position: Position :math:`s_x`- and :math:`s_y` in a global coordinate system
    :param steering_angle: Steering angle :math:`\\delta`
    :param velocity: Velocity :math:`v_x` in longitudinal direction
    :param orientation: Yaw angle :math:`\\Psi`
    """
    position: ExactOrShape = None
    steering_angle: FloatExactOrInterval = None
    velocity: FloatExactOrInterval = None
    orientation: AngleExactOrInterval = None


@dataclass(eq=False)
class KSTState(KSState):
    """
    This is a class representing Kinematic Single Track State (KST State).

    :param hitch_angle: Hitch angle :math:`\\alpha`
    """
    hitch_angle: AngleExactOrInterval = None


@dataclass(eq=False)
class STState(KSState):
    """
    This is a class representing Single Track State (ST State).

    :param slip_angle: Slip angle :math:`\\beta`
    :param yaw_rate: Yaw rate :math:`\\dot{\\Psi}`
    """
    slip_angle: FloatExactOrInterval = None
    yaw_rate: FloatExactOrInterval = None


@dataclass(eq=False)
class STDState(STState):
    """
    This is a class representing Single Track Drift State (STD State).

    :param front_wheel_angular_speed: Front wheel angular speed :math:`\\omega_{f}`
    :param rear_wheel_angular_speed: Rear wheel angular speed :math:`\\omega_{r}`
    """
    front_wheel_angular_speed: FloatExactOrInterval = None
    rear_wheel_angular_speed: FloatExactOrInterval = None


@dataclass(eq=False)
class MBState(State):
    """
    This is a class representing Multi Body State (MB State).

    :param position: Position :math:`s_x`- and :math:`s_y` in a global coordinate system
    :param steering_angle: Steering angle :math:`\\delta`
    :param velocity: Velocity :math:`v_x` in longitudinal direction
    :param orientation: Yaw angle :math:`\\Psi`
    :param yaw_rate: Yaw rate :math:`\\dot{\\Psi}`
    :param roll_angle: Roll angle :math:`\\Phi_S`
    :param roll_rate: Roll rate :math:`\\dot{\\Phi}_S`
    :param pitch_angle: Pitch angle :math:`\\Theta_S`
    :param pitch_rate: Pitch rate :math:`\\dot{\\Theta}_S`
    :param velocity_y: Velocity :math:`v_y` in lateral direction
    :param position_z: Position :math:`s_z` (height) from ground
    :param velocity_z: Velocity :math:`v_z` in vertical direction perpendicular to road plane
    :param roll_angle_front: Roll angle front :math:`\\Phi_{UF}`
    :param roll_rate_front: Roll rate front :math:`\\dot{\\Phi}_{UF}`
    :param velocity_y_front: Velocity :math:`v_{y,UF}` in y-direction front
    :param position_z_front: Position :math:`s_{z,UF}` in z-direction front
    :param velocity_z_front: Velocity :math:`v_{z,UF}` in z-direction front
    :param roll_angle_rear: Roll angle rear :math:`\\Phi_{UR}`
    :param roll_rate_rear: Roll rate rear :math:`\\dot{\\Phi}_{UR}`
    :param velocity_y_rear: Velocity :math:`v_{y,UR}` in y-direction rear
    :param position_z_rear: Position :math:`s_{z,UR}` in z-direction rear
    :param velocity_z_rear: Velocity :math:`v_{z,UR}` in z-direction rear
    :param left_front_wheel_angular_speed: Left front wheel angular speed :math:`\\omega_{LF}`
    :param right_front_wheel_angular_speed: Right front wheel angular speed :math:`\\omega_{RF}`
    :param left_rear_wheel_angular_speed: Left rear wheel angular speed :math:`\\omega_{LR}`
    :param right_rear_wheel_angular_speed: Right rear wheel angular speed :math:`\\omega_{RR}`
    :param delta_y_f: Front lateral displacement :math:`\\delta_{y,f}` of sprung mass due to roll
    :param delta_y_r: Rear lateral displacement :math:`\\delta_{y,r}` of sprung mass due to roll
    """
    position: ExactOrShape = None
    steering_angle: FloatExactOrInterval = None
    velocity: FloatExactOrInterval = None
    orientation: AngleExactOrInterval = None
    yaw_rate: FloatExactOrInterval = None
    roll_angle: FloatExactOrInterval = None
    roll_rate: FloatExactOrInterval = None
    pitch_angle: FloatExactOrInterval = None
    pitch_rate: FloatExactOrInterval = None
    velocity_y: FloatExactOrInterval = None
    position_z: FloatExactOrInterval = None
    velocity_z: FloatExactOrInterval = None
    roll_angle_front: FloatExactOrInterval = None
    roll_rate_front: FloatExactOrInterval = None
    velocity_y_front: FloatExactOrInterval = None
    position_z_front: FloatExactOrInterval = None
    velocity_z_front: FloatExactOrInterval = None
    roll_angle_rear: FloatExactOrInterval = None
    roll_rate_rear: FloatExactOrInterval = None
    velocity_y_rear: FloatExactOrInterval = None
    position_z_rear: FloatExactOrInterval = None
    velocity_z_rear: FloatExactOrInterval = None
    left_front_wheel_angular_speed: FloatExactOrInterval = None
    right_front_wheel_angular_speed: FloatExactOrInterval = None
    left_rear_wheel_angular_speed: FloatExactOrInterval = None
    right_rear_wheel_angular_speed: FloatExactOrInterval = None
    delta_y_f: FloatExactOrInterval = None
    delta_y_r: FloatExactOrInterval = None


@dataclass(eq=False)
class LongitudinalState(State):
    """
    This is a class representing the Longitudinal Motion State (Longitudinal State). The state cannot be read by
    the file reader because neither orientation nor velocity in y direction are specified.

    :param longitudinal_position: Longitudinal position :math:`s` along reference path
    :param velocity: Velocity :math:`v_x` in longitudinal direction
    :param acceleration: Acceleration :math:`a_x` in longitudinal direction
    :param jerk: Jerk :math:`j`
    """
    longitudinal_position: FloatExactOrInterval = None
    velocity: FloatExactOrInterval = None
    acceleration: FloatExactOrInterval = None
    jerk: FloatExactOrInterval = None


@dataclass(eq=False)
class LateralState(State):
    """
    This is a class representing the Lateral Motion State (Lateral State). The state cannot be read by the file reader
    because position is not specified.

    :param lateral_position: Lateral distance :math:`d` to reference path
    :param orientation: Yaw angle :math:`\\Psi`
    :param curvature: Curvature math:`\\kappa`
    :param curvature_rate: Change of curvature math:`\\dot{\\kappa}`
    """
    lateral_position: FloatExactOrInterval = None
    orientation: AngleExactOrInterval = None
    curvature: FloatExactOrInterval = None
    curvature_rate: FloatExactOrInterval = None


@dataclass(eq=False)
class InputState(State):
    """
    This is a class representing the input for most supported models.

    :param steering_angle_speed: Steering angle speed :math:`\\dot{\\delta}` of front wheels
    :param acceleration: Acceleration :math:`a_x`
    """
    steering_angle_speed: FloatExactOrInterval = None
    acceleration: FloatExactOrInterval = None


@dataclass(eq=False)
class PMInputState(State):
    """
    This is a class representing the input for PM model (PM Input).

    :param acceleration: Acceleration :math:`a_x`
    :param acceleration_y: Acceleration :math:`a_y`
    """
    acceleration: FloatExactOrInterval = None
    acceleration_y: FloatExactOrInterval = None


@dataclass(eq=False)
class LKSInputState(State):
    """
    This is a class representing the input for the linearized kinematic single track (LKS) model (LKS Input).

    :param longitudinal snap: change of longitudinal jerk (i.e., jerk_dot): math:`\\dot{j}`
    :param curvature rate rate: change of curvature rate (i.e., kappa_dot_dot)  :math:`\\ddot{\\kappa}`
    """
    jerk_dot: FloatExactOrInterval = None
    kappa_dot_dot: FloatExactOrInterval = None


class CustomState(State):
    """
    This is a class representing the custom state. State variables can be added at runtime. The attributes position
    and orientation/velocity_y are necessary for successful file reading.
    """

    def __init__(self, **attributes):
        """
        Additional constructor for CustomState class.

        :param attributes: Variable number of attributes each consisting of name and value.
        """
        if len(attributes) > 0:  # if one wants to use the attribute adding methods manually
            super().__init__(attributes["time_step"])
        for name, value in attributes.items():
            if name == "time_step":
                continue
            self.add_attribute(name)
            self.set_value(name, value)

    def add_attribute(self, new_attr: str):
        """
        Adds a new attribute to custom state.

        :param new_attr: Attribute name
        """
        setattr(self, new_attr, None)

    def set_value(self, attr: str, value: Any):
        """
        Sets value to attribute.

        :param attr: Attribute name
        :param value: Value
        """
        assert attr in self.attributes, "{} is not an attribute of this custom state!".format(attr)

        setattr(self, attr, value)


TraceState = Union[State, InitialState, PMState, KSState, KSTState, STState, STDState, MBState, InputState,
                   PMInputState, LateralState, LongitudinalState, CustomState]

SpecificStateClasses = [InitialState, PMState, KSState, KSTState, STState, STDState, MBState, InputState,
                        PMInputState, LateralState, LongitudinalState]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: traffic_light.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.util_pb2 as util__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='traffic_light.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13traffic_light.proto\x12\ncommonroad\x1a\nutil.proto\"j\n\x15TrafficLightStateEnum\"Q\n\x11TrafficLightState\x12\x07\n\x03RED\x10\x00\x12\x0e\n\nRED_YELLOW\x10\x01\x12\t\n\x05GREEN\x10\x02\x12\n\n\x06YELLOW\x10\x03\x12\x0c\n\x08INACTIVE\x10\x04\"\x97\x01\n\x19TrafficLightDirectionEnum\"z\n\x15TrafficLightDirection\x12\t\n\x05RIGHT\x10\x00\x12\x0c\n\x08STRAIGHT\x10\x01\x12\x08\n\x04LEFT\x10\x02\x12\x11\n\rLEFT_STRAIGHT\x10\x03\x12\x12\n\x0eSTRAIGHT_RIGHT\x10\x04\x12\x0e\n\nLEFT_RIGHT\x10\x05\x12\x07\n\x03\x41LL\x10\x06\"d\n\x0c\x43ycleElement\x12\x10\n\x08\x64uration\x18\x01 \x02(\r\x12\x42\n\x05\x63olor\x18\x02 \x02(\x0e\x32\x33.commonroad.TrafficLightStateEnum.TrafficLightState\"\xf4\x01\n\x0cTrafficLight\x12\x18\n\x10traffic_light_id\x18\x01 \x02(\r\x12\x30\n\x0e\x63ycle_elements\x18\x02 \x03(\x0b\x32\x18.commonroad.CycleElement\x12#\n\x08position\x18\x03 \x01(\x0b\x32\x11.commonroad.Point\x12\x13\n\x0btime_offset\x18\x04 \x01(\r\x12N\n\tdirection\x18\x05 \x01(\x0e\x32;.commonroad.TrafficLightDirectionEnum.TrafficLightDirection\x12\x0e\n\x06\x61\x63tive\x18\x06 \x01(\x08'
  ,
  dependencies=[util__pb2.DESCRIPTOR,])



_TRAFFICLIGHTSTATEENUM_TRAFFICLIGHTSTATE = _descriptor.EnumDescriptor(
  name='TrafficLightState',
  full_name='commonroad.TrafficLightStateEnum.TrafficLightState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RED_YELLOW', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='GREEN', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='YELLOW', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='INACTIVE', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=72,
  serialized_end=153,
)
_sym_db.RegisterEnumDescriptor(_TRAFFICLIGHTSTATEENUM_TRAFFICLIGHTSTATE)

_TRAFFICLIGHTDIRECTIONENUM_TRAFFICLIGHTDIRECTION = _descriptor.EnumDescriptor(
  name='TrafficLightDirection',
  full_name='commonroad.TrafficLightDirectionEnum.TrafficLightDirection',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RIGHT', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRAIGHT', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEFT', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEFT_STRAIGHT', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STRAIGHT_RIGHT', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='LEFT_RIGHT', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ALL', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=185,
  serialized_end=307,
)
_sym_db.RegisterEnumDescriptor(_TRAFFICLIGHTDIRECTIONENUM_TRAFFICLIGHTDIRECTION)


_TRAFFICLIGHTSTATEENUM = _descriptor.Descriptor(
  name='TrafficLightStateEnum',
  full_name='commonroad.TrafficLightStateEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TRAFFICLIGHTSTATEENUM_TRAFFICLIGHTSTATE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=47,
  serialized_end=153,
)


_TRAFFICLIGHTDIRECTIONENUM = _descriptor.Descriptor(
  name='TrafficLightDirectionEnum',
  full_name='commonroad.TrafficLightDirectionEnum',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _TRAFFICLIGHTDIRECTIONENUM_TRAFFICLIGHTDIRECTION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=156,
  serialized_end=307,
)


_CYCLEELEMENT = _descriptor.Descriptor(
  name='CycleElement',
  full_name='commonroad.CycleElement',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='duration', full_name='commonroad.CycleElement.duration', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='color', full_name='commonroad.CycleElement.color', index=1,
      number=2, type=14, cpp_type=8, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=309,
  serialized_end=409,
)


_TRAFFICLIGHT = _descriptor.Descriptor(
  name='TrafficLight',
  full_name='commonroad.TrafficLight',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='traffic_light_id', full_name='commonroad.TrafficLight.traffic_light_id', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cycle_elements', full_name='commonroad.TrafficLight.cycle_elements', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='position', full_name='commonroad.TrafficLight.position', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_offset', full_name='commonroad.TrafficLight.time_offset', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='direction', full_name='commonroad.TrafficLight.direction', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='active', full_name='commonroad.TrafficLight.active', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=412,
  serialized_end=656,
)

_TRAFFICLIGHTSTATEENUM_TRAFFICLIGHTSTATE.containing_type = _TRAFFICLIGHTSTATEENUM
_TRAFFICLIGHTDIRECTIONENUM_TRAFFICLIGHTDIRECTION.containing_type = _TRAFFICLIGHTDIRECTIONENUM
_CYCLEELEMENT.fields_by_name['color'].enum_type = _TRAFFICLIGHTSTATEENUM_TRAFFICLIGHTSTATE
_TRAFFICLIGHT.fields_by_name['cycle_elements'].message_type = _CYCLEELEMENT
_TRAFFICLIGHT.fields_by_name['position'].message_type = util__pb2._POINT
_TRAFFICLIGHT.fields_by_name['direction'].enum_type = _TRAFFICLIGHTDIRECTIONENUM_TRAFFICLIGHTDIRECTION
DESCRIPTOR.message_types_by_name['TrafficLightStateEnum'] = _TRAFFICLIGHTSTATEENUM
DESCRIPTOR.message_types_by_name['TrafficLightDirectionEnum'] = _TRAFFICLIGHTDIRECTIONENUM
DESCRIPTOR.message_types_by_name['CycleElement'] = _CYCLEELEMENT
DESCRIPTOR.message_types_by_name['TrafficLight'] = _TRAFFICLIGHT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TrafficLightStateEnum = _reflection.GeneratedProtocolMessageType('TrafficLightStateEnum', (_message.Message,), {
  'DESCRIPTOR' : _TRAFFICLIGHTSTATEENUM,
  '__module__' : 'traffic_light_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.TrafficLightStateEnum)
  })
_sym_db.RegisterMessage(TrafficLightStateEnum)

TrafficLightDirectionEnum = _reflection.GeneratedProtocolMessageType('TrafficLightDirectionEnum', (_message.Message,), {
  'DESCRIPTOR' : _TRAFFICLIGHTDIRECTIONENUM,
  '__module__' : 'traffic_light_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.TrafficLightDirectionEnum)
  })
_sym_db.RegisterMessage(TrafficLightDirectionEnum)

CycleElement = _reflection.GeneratedProtocolMessageType('CycleElement', (_message.Message,), {
  'DESCRIPTOR' : _CYCLEELEMENT,
  '__module__' : 'traffic_light_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.CycleElement)
  })
_sym_db.RegisterMessage(CycleElement)

TrafficLight = _reflection.GeneratedProtocolMessageType('TrafficLight', (_message.Message,), {
  'DESCRIPTOR' : _TRAFFICLIGHT,
  '__module__' : 'traffic_light_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.TrafficLight)
  })
_sym_db.RegisterMessage(TrafficLight)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: commonroad.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import commonroad.scenario_definition.protobuf_format.generated_scripts.util_pb2 as util__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.scenario_tags_pb2 as scenario__tags__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.location_pb2 as location__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.lanelet_pb2 as lanelet__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.traffic_sign_pb2 as traffic__sign__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.traffic_light_pb2 as traffic__light__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.intersection_pb2 as intersection__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.static_obstacle_pb2 as static__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.dynamic_obstacle_pb2 as dynamic__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.environment_obstacle_pb2 as environment__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.phantom_obstacle_pb2 as phantom__obstacle__pb2
import commonroad.scenario_definition.protobuf_format.generated_scripts.planning_problem_pb2 as planning__problem__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='commonroad.proto',
  package='commonroad',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10\x63ommonroad.proto\x12\ncommonroad\x1a\nutil.proto\x1a\x13scenario_tags.proto\x1a\x0elocation.proto\x1a\rlanelet.proto\x1a\x12traffic_sign.proto\x1a\x13traffic_light.proto\x1a\x12intersection.proto\x1a\x15static_obstacle.proto\x1a\x16\x64ynamic_obstacle.proto\x1a\x1a\x65nvironment_obstacle.proto\x1a\x16phantom_obstacle.proto\x1a\x16planning_problem.proto\"\xba\x01\n\x13ScenarioInformation\x12\x1b\n\x13\x63ommon_road_version\x18\x01 \x02(\t\x12\x14\n\x0c\x62\x65nchmark_id\x18\x02 \x02(\t\x12#\n\x04\x64\x61te\x18\x03 \x02(\x0b\x32\x15.commonroad.TimeStamp\x12\x0e\n\x06\x61uthor\x18\x04 \x02(\t\x12\x13\n\x0b\x61\x66\x66iliation\x18\x05 \x02(\t\x12\x0e\n\x06source\x18\x06 \x02(\t\x12\x16\n\x0etime_step_size\x18\x07 \x02(\x01\"\xf3\x04\n\nCommonRoad\x12\x34\n\x0binformation\x18\x01 \x02(\x0b\x32\x1f.commonroad.ScenarioInformation\x12/\n\rscenario_tags\x18\x02 \x02(\x0b\x32\x18.commonroad.ScenarioTags\x12&\n\x08location\x18\x03 \x02(\x0b\x32\x14.commonroad.Location\x12%\n\x08lanelets\x18\x04 \x03(\x0b\x32\x13.commonroad.Lanelet\x12.\n\rtraffic_signs\x18\x05 \x03(\x0b\x32\x17.commonroad.TrafficSign\x12\x30\n\x0etraffic_lights\x18\x06 \x03(\x0b\x32\x18.commonroad.TrafficLight\x12/\n\rintersections\x18\x07 \x03(\x0b\x32\x18.commonroad.Intersection\x12\x34\n\x10static_obstacles\x18\x08 \x03(\x0b\x32\x1a.commonroad.StaticObstacle\x12\x36\n\x11\x64ynamic_obstacles\x18\t \x03(\x0b\x32\x1b.commonroad.DynamicObstacle\x12>\n\x15\x65nvironment_obstacles\x18\n \x03(\x0b\x32\x1f.commonroad.EnvironmentObstacle\x12\x36\n\x11phantom_obstacles\x18\x0b \x03(\x0b\x32\x1b.commonroad.PhantomObstacle\x12\x36\n\x11planning_problems\x18\x0c \x03(\x0b\x32\x1b.commonroad.PlanningProblem'
  ,
  dependencies=[util__pb2.DESCRIPTOR,scenario__tags__pb2.DESCRIPTOR,location__pb2.DESCRIPTOR,lanelet__pb2.DESCRIPTOR,traffic__sign__pb2.DESCRIPTOR,traffic__light__pb2.DESCRIPTOR,intersection__pb2.DESCRIPTOR,static__obstacle__pb2.DESCRIPTOR,dynamic__obstacle__pb2.DESCRIPTOR,environment__obstacle__pb2.DESCRIPTOR,phantom__obstacle__pb2.DESCRIPTOR,planning__problem__pb2.DESCRIPTOR,])




_SCENARIOINFORMATION = _descriptor.Descriptor(
  name='ScenarioInformation',
  full_name='commonroad.ScenarioInformation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='common_road_version', full_name='commonroad.ScenarioInformation.common_road_version', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='benchmark_id', full_name='commonroad.ScenarioInformation.benchmark_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='date', full_name='commonroad.ScenarioInformation.date', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='author', full_name='commonroad.ScenarioInformation.author', index=3,
      number=4, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='affiliation', full_name='commonroad.ScenarioInformation.affiliation', index=4,
      number=5, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source', full_name='commonroad.ScenarioInformation.source', index=5,
      number=6, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='time_step_size', full_name='commonroad.ScenarioInformation.time_step_size', index=6,
      number=7, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=281,
  serialized_end=467,
)


_COMMONROAD = _descriptor.Descriptor(
  name='CommonRoad',
  full_name='commonroad.CommonRoad',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='information', full_name='commonroad.CommonRoad.information', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scenario_tags', full_name='commonroad.CommonRoad.scenario_tags', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='commonroad.CommonRoad.location', index=2,
      number=3, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='lanelets', full_name='commonroad.CommonRoad.lanelets', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_signs', full_name='commonroad.CommonRoad.traffic_signs', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='traffic_lights', full_name='commonroad.CommonRoad.traffic_lights', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='intersections', full_name='commonroad.CommonRoad.intersections', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='static_obstacles', full_name='commonroad.CommonRoad.static_obstacles', index=7,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dynamic_obstacles', full_name='commonroad.CommonRoad.dynamic_obstacles', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='environment_obstacles', full_name='commonroad.CommonRoad.environment_obstacles', index=9,
      number=10, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phantom_obstacles', full_name='commonroad.CommonRoad.phantom_obstacles', index=10,
      number=11, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='planning_problems', full_name='commonroad.CommonRoad.planning_problems', index=11,
      number=12, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=470,
  serialized_end=1097,
)

_SCENARIOINFORMATION.fields_by_name['date'].message_type = util__pb2._TIMESTAMP
_COMMONROAD.fields_by_name['information'].message_type = _SCENARIOINFORMATION
_COMMONROAD.fields_by_name['scenario_tags'].message_type = scenario__tags__pb2._SCENARIOTAGS
_COMMONROAD.fields_by_name['location'].message_type = location__pb2._LOCATION
_COMMONROAD.fields_by_name['lanelets'].message_type = lanelet__pb2._LANELET
_COMMONROAD.fields_by_name['traffic_signs'].message_type = traffic__sign__pb2._TRAFFICSIGN
_COMMONROAD.fields_by_name['traffic_lights'].message_type = traffic__light__pb2._TRAFFICLIGHT
_COMMONROAD.fields_by_name['intersections'].message_type = intersection__pb2._INTERSECTION
_COMMONROAD.fields_by_name['static_obstacles'].message_type = static__obstacle__pb2._STATICOBSTACLE
_COMMONROAD.fields_by_name['dynamic_obstacles'].message_type = dynamic__obstacle__pb2._DYNAMICOBSTACLE
_COMMONROAD.fields_by_name['environment_obstacles'].message_type = environment__obstacle__pb2._ENVIRONMENTOBSTACLE
_COMMONROAD.fields_by_name['phantom_obstacles'].message_type = phantom__obstacle__pb2._PHANTOMOBSTACLE
_COMMONROAD.fields_by_name['planning_problems'].message_type = planning__problem__pb2._PLANNINGPROBLEM
DESCRIPTOR.message_types_by_name['ScenarioInformation'] = _SCENARIOINFORMATION
DESCRIPTOR.message_types_by_name['CommonRoad'] = _COMMONROAD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ScenarioInformation = _reflection.GeneratedProtocolMessageType('ScenarioInformation', (_message.Message,), {
  'DESCRIPTOR' : _SCENARIOINFORMATION,
  '__module__' : 'commonroad_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.ScenarioInformation)
  })
_sym_db.RegisterMessage(ScenarioInformation)

CommonRoad = _reflection.GeneratedProtocolMessageType('CommonRoad', (_message.Message,), {
  'DESCRIPTOR' : _COMMONROAD,
  '__module__' : 'commonroad_pb2'
  # @@protoc_insertion_point(class_scope:commonroad.CommonRoad)
  })
_sym_db.RegisterMessage(CommonRoad)


# @@protoc_insertion_point(module_scope)
